local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

local resourceCopperBuyPriceDefault = 0

local resourceCopperCostDefault = 100 * 10000

local costsMemory = { }

function Capitalist:CostMemoryReset()
	self:DebugMisc("CostMemoryReset")
	costsMemory = { }
end

--
--	Value:	vendor sell value, and then inflated to value as a
--			subcomponent of a product that has vendor sell value
--
--  for          resource, Value <= Buy Price <= Cost
--	for merchant resource, Value  = Buy Price  = Cost
--

function Capitalist:HasComponentCopperValue(componentName, value)
	return (self:IsComponent(componentName) and
			(self.db.profile.components[componentName].value ~= nil))
end

function Capitalist:GetComponentCopperValue(componentName, value)
	if (self:IsComponent(componentName)) then
		return self.db.profile.components[componentName].value
	end
	return nil
end

-- REVIEW: only the 'nil' value cases call this; should use it for the other cases too
function Capitalist:SetComponentCopperValue(componentName, value, valueSrc)
	if (self:IsComponent(componentName)) then
		self.db.profile.components[componentName].value = value
		self.db.profile.components[componentName].valueSrc = valueSrc
	end
end

--
--	Buy Price:	for non-merchant-supplied resources, allows for
--              user to specify an "accumulate" price for a resource
--
--  for          resource, Value <= Buy Price <= Cost
--	for merchant resource, Value  = Buy Price  = Cost
--

function Capitalist:HasResourceCopperBuyPrice(resourceName, buyPrice)
	return (self:IsResource(resourceName) and
			(self.db.profile.components[resourceName].buyPrice ~= nil))
end

function Capitalist:GetResourceCopperBuyPrice(resourceName, buyPrice)
	local buyPrice
	if (self:IsResource(resourceName)) then
		buyPrice = self.db.profile.components[resourceName].buyPrice
		if (not buyPrice) then
			buyPrice = resourceCopperBuyPriceDefault
		end
	end
	return buyPrice
end

function Capitalist:SetResourceCopperBuyPrice(resourceName, buyPrice)
	if (self:IsResource(resourceName)) then
		self.db.profile.components[resourceName].buyPrice = buyPrice
	end
end

--
--	Cost:	for non-merchant-supplied resources, allows for
--          user to specify an "assumed cost" for the resource
--			used in calculating the "cost" of a product made
--          from that resource
--
--  for          resource, Value <= Buy Price <= Cost
--	for merchant resource, Value  = Buy Price  = Cost
--

function Capitalist:HasResourceCopperCost(resourceName)
	if (self.db.profile.components and
		self.db.profile.components[resourceName] and
		self.db.profile.components[resourceName].cost) then
		return true
	end
	return false
end

function Capitalist:GetResourceCopperCost(resourceName)
	if (self.db.profile.components and
		self.db.profile.components[resourceName] and
		self.db.profile.components[resourceName].cost) then
		return self.db.profile.components[resourceName].cost
	elseif (self:ResourceGetMerchant(resourceName)) then
		return self:GetResourceCopperBuyPrice(resourceName)
	else
		return resourceCopperCostDefault
	end
end

function Capitalist:SetResourceCopperCost(resourceName, cost)
	self.db.profile.components[resourceName].cost = cost
end

--
-- Functions that adjust Value, Buy Price, and Cost based on
-- analysis across products and components
--

function Capitalist:ClearValues()
	self:Print(L["Clearing merchant valuations"])
	self:LogClearCategory("value")
	if (self.db.profile.components) then
		for componentName, componentCount in pairs(self.db.profile.components) do
			self:DebugMisc(componentName)
			self:SetComponentCopperValue(componentName, nil)
		end
	end
end

function Capitalist:CalcValues()
	if (self.db.profile.enableMerchantCosting and self.db.profile.enableMerchantValuation) then
		self:DebugMisc("UpdateComponentValues")
		if (self:UpdateComponentValues()) then
			self:DebugMisc("  delta")
		end

		self:DebugMisc("UpdateCombinedValues")
		if (self:UpdateCombinedValues()) then
			self:DebugMisc("  delta")
		end

		--TODO: remove after debugging
		self:EnforceProductsCannotHaveCostSetting()
	end
end

-- For each known component:
-- 1. if it has a 'link', but no 'value':
--		a. but has a link field:
--		b. extract the item id
--		c. obtain itemSellPrice via GetItemInfo
--		d. update value
-- 2. if it has 'value' > 'buyPrice', remove 'buyPrice'
--    (i.e. set it to the same and then save memory)
function Capitalist:UpdateComponentValues()
	local dataChanged = false

	if (self.db.profile.enableMerchantCosting and self.db.profile.enableMerchantValuation) then
		if (self.db.profile.components) then
			for componentName, componentData in pairs(self.db.profile.components) do
				if (componentData.link and (not componentData.value)) then
					local itemIdString = string.match(componentData.link, "Hitem:(%d+)")
					local itemIdNumber = tonumber(itemIdString)

					local _, _, _, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo(itemIdNumber)
					if (itemSellPrice) then
						self:DebugMisc(componentName)
						componentData.value = itemSellPrice
						componentData.valueSrc = "merchant"
						dataChanged = true
					end
				end

				local value = componentData.value
				local buyPrice = componentData.buyPrice
				if (value and buyPrice and (value > buyPrice)) then
					self:DebugMisc(componentName)
					componentData.buyPrice = nil
					dataChanged = true
				end
			end
		end
	end

	return dataChanged
end

-- For an individual product that has a 'sellPrice':
-- 1. set newComponentValue to the product sellPrice
-- 2. enumerate through components:
--		a. if it has a 'merchant' and 'buyPrice', then discount newComponentValue
--		   by the buyPriceIterator * componentCountIterator for that component and loop
--		b. if local componentName is set or componentNameIterator is a product,
--		   return from the function (we can't deduce any new value for a component)
--		c. if local componentName is not set, set componentName and componentCount
--
-- If we survived 2b and have a componentName, then we can deduce it's value as
-- a contributor to this product's sellPrice.
-- 3. multiply newComponentValue by the 'min' value of this skill (i.e. the effective
--    value of this component is enhanced by the fact that the skill makes multiple)
-- 4. divide the newComponentValue by the componentCount required
--
-- At this point we've arrived at the value of this component as part of this skill
-- if the product(s) produced are sold to a merchant.
-- 5. compare newComponentValue with the existing 'value' previously stored in the DB
-- 6. if newComponentValue is higher:
--		a. if this is a merchant-supplied resource, spit out a "ZOMG print money" message
--		b. if not, update the 'value' for this component to newComponentValue and update
--		   dataChanged so that the caller knows something changed in the matrix
local N = 0
function Capitalist:UpdateProductComponentValues(profession, professionData, skillName)
	local componentChanged
	local componentUnchanged
	local increment = 0
	local componentsData = self.db.profile.components
	local product = professionData[skillName]
	local sellPrice = product.sellPrice

	local logEntry = {}
	logEntry.profession = profession
	logEntry.product = product
	logEntry.skillName = skillName
	logEntry.sellPrice = sellPrice

	if (sellPrice) then
		local newComponentValue = sellPrice

		logEntry.components = {}

		local componentName
		local componentCount
		for componentNameIterator, componentCountIterator in pairs(product.components) do
			local componentDataIterator = componentsData[componentNameIterator]

			logEntry.components[componentNameIterator] = { componentDataIterator = componentDataIterator, }

			local buyPriceIterator
			if (componentDataIterator.merchant) then
				buyPriceIterator = componentDataIterator.buyPrice
			end

			if ((not buyPriceIterator) and self:IsProduct(componentNameIterator)) then
				-- TODO: parts value, if known
				-- buyPriceIterator =
			end

			logEntry.components[componentNameIterator].buyPriceIterator = buyPriceIterator

			if (buyPriceIterator) then
				local excludedComponentContribution = buyPriceIterator * componentCountIterator / (product.min or 1)
				newComponentValue = newComponentValue - excludedComponentContribution

				logEntry.components[componentNameIterator].excludedComponentContribution = excludedComponentContribution
				logEntry.components[componentNameIterator].newComponentValue = newComponentValue
			else
				if (componentName) then
					-- already have a candidate, can't have two
					logEntry.components[componentNameIterator].secondCandidate = true
					return
				else
					-- don't have a candidate, and this one does qualify
					componentName = componentNameIterator
					componentCount = componentCountIterator
					logEntry.components[componentNameIterator].componentName = componentName
					logEntry.components[componentNameIterator].componentCount = componentCount
				end
			end
		end

		if (componentName) then
			logEntry.componentName = componentName

			if (product.min) then
				newComponentValue = newComponentValue * product.min
				logEntry.newComponentValue2 = newComponentValue
			end
			newComponentValue = math.floor(newComponentValue / componentCount)
			logEntry.newComponentValue3 = newComponentValue

			-- round down to the nearest copper
			newComponentValue = math.floor(newComponentValue)
			logEntry.newComponentValue4 = newComponentValue

			local oldComponentValue = self:GetComponentCopperValue(componentName) or 0
			logEntry.oldComponentValue = oldComponentValue

			componentUnchanged = componentName
			logEntry.componentUnchanged = componentUnchanged

			-- The value increase of the component must be in whole copper increments
			if (newComponentValue - oldComponentValue >= componentCount) then
				local newBuyString = self:PrettyCurrency(newComponentValue)
				local oldBuyString = self:PrettyCurrency(oldComponentValue)
				logEntry.newBuyString = newBuyString
				logEntry.oldBuyString = oldBuyString

				local isMerchantResource = (self:ResourceGetMerchant(componentName) ~= nil)
				logEntry.isMerchantResource = isMerchantResource
				if (isMerchantResource) then
					-- REVIEW: localize or DebugMisc
					self:Print(string.format("MERCHANT PROFIT OPPORTUNITY: skill %s, component %s.", skillName, componentName))
				else
					local skillText = self:SkillTextFromSkillName(profession, skillName)
					self:Print(
						string.format(
							"  "..L["%s: Updating %s value from %s to %s."],
							skillText, self:GetComponentText(componentName), oldBuyString, newBuyString
						))
					logEntry.skillText = skillText

					self.db.profile.components[componentName].value = newComponentValue
					self.db.profile.components[componentName].valueSrc = skillName
					logEntry.value = newComponentValue
					logEntry.valueSrc = skillName

					componentChanged = componentName
					componentUnchanged = nil
					increment = newComponentValue - oldComponentValue
					logEntry.commentChanged = commentChanged
					logEntry.commentUnchanged = commentUnchanged
					logEntry.increment = increment
				end
			end
		end
	end

	self:LogCreateEntry("value", logEntry)

	return componentChanged, componentUnchanged, increment
end

local PC = 0
local PP = 0
local CTP = 0
local UV = 0
local AC = 0
function Capitalist:UpdateCombinedValuesSkillComponentProcessed(	pendingComponents, pendingProducts, adjustableComponents,
																	profession, skillName, componentName, componentValueIncrement)

	if (componentValueIncrement > 0) then
		for profession, pendingProfessionData in pairs(pendingComponents[componentName]) do
			for skillName, pendingSkillData in pairs(pendingProfessionData) do
				local deltaComponentValueContribution = self.db.profile.professions[profession][skillName].components[componentName] * componentValueIncrement
				pendingProducts[profession][skillName].unclaimedValue = 
					pendingProducts[profession][skillName].unclaimedValue - deltaComponentValueContribution
			end
		end
	end

	if (pendingComponents[componentName] and
		pendingComponents[componentName][profession] and
		pendingComponents[componentName][profession][skillName]) then
		pendingComponents[componentName][profession][skillName] = nil

		local pendingComponentProfessionHasSkills
		for skillName, componentCount in pairs(pendingComponents[componentName][profession]) do
			pendingComponentProfessionHasSkills = true
			break
		end
		if (not pendingComponentProfessionHasSkills) then
			pendingComponents[componentName][profession] = nil
			CTP = CTP - 1
		end
		local pendingComponentHasProfessions
		for profession, professionData in pairs(pendingComponents[componentName]) do
			pendingComponentHasProfessions = true
			break
		end
		if (not pendingComponentHasProfessions) then
			pendingComponents[componentName] = nil
			PC = PC - 1
		end

		adjustableComponents[profession][skillName] = adjustableComponents[profession][skillName] - 1

		if (adjustableComponents[profession][skillName] == 0) then
			adjustableComponents[profession][skillName] = nil
			AC=AC-1

			pendingProducts[profession][skillName] = nil
			PP = PP - 1
			UV = UV - 1
		end
	end
end

-- TODO: VERSION2 COMMENT NEEDS UPDATING
-- Iterate across all profession, all skills, in multiple passes as long as changes are
-- being made.  Leverage UpdateProductComponentValues [above] to repeatedly refine the
-- 'value' of the components when used to craft products for selling to merchants.
--
-- You might think "but merchant prices are so low".  Yes, they are.  The point of this
-- algorithm is to find the minimum value of components.  With this information, we can
-- safely and aggressively accumulate these components knowing that -- at worst -- the
-- products can be sold to merchants if the AH buyer market is soft.
function Capitalist:UpdateCombinedValues()
	self:DebugEntry("UpdateCombinedValues")

	if (not (self.db.profile.enableMerchantCosting and self.db.profile.enableMerchantValuation)) then
		return self:DebugExit("UpdateCombinedValues", false)
	end

	self:DebugMisc(time())
	local dataChanged = false

	if (self.db.profile.components and self.db.profile.professions) then
		-- components that can still grow in value
		--		pendingComponents[componentName] implies component is still adjustable
		--		pendingComponents[componentName][profession] implies it used by a skill in that profession
		--		pendingComponents[componentName][profession][skillName] implies it is used by that skill
		--		pendingComponents[componentName][profession][skillName] == count of component used by the skill
		local pendingComponents = {}

		-- products that haven't had all value distributed
		local pendingProducts = {}

		-- for products in pendingProducts, how many adjustable components are there
		local adjustableComponents = {}

PC = 0
		-- 1. add all components that are not merchant resources or have no buyPrice set
		for componentName, componentData in pairs(self.db.profile.components) do
			if (not (componentData.merchant and componentData.buyPrice and (componentData.buyPrice > 0))) then
				pendingComponents[componentName] = {}
				PC = PC + 1
			end
		end

PP = 0
		-- 2. add all products
		for profession, professionData in pairs(self.db.profile.professions) do
			pendingProducts[profession] = {}
			for skillName, skillData in pairs(professionData) do
				pendingProducts[profession][skillName] = {}
				PP = PP + 1
			end
		end

CTP = 0
UV = 0
AC = 0
		-- 3. populate pendingComponents with associated skills and update unclaimedValue
		for profession, professionData in pairs(self.db.profile.professions) do
			adjustableComponents[profession] = {}

			for skillName, pendingProductData in pairs(pendingProducts[profession]) do
				local product = professionData[skillName]

				if (not product.sellPrice) then
					-- a. if no sell price, product is done processing
					pendingProducts[profession][skillName] = nil
				else
					-- b. unclaimed value is product sell price (to merchant) * minimum produced by using the skill
					local unclaimedComponentValue = product.sellPrice
					if (product.min) then
						unclaimedComponentValue = unclaimedComponentValue * product.min
					end

					--TODO
					-- c. trim components with a fixed value

					-- c. subtract off the current component values
					for componentName, componentCount in pairs(product.components) do
						--TODO
						--local componentValueContribution = componentCount * self.db.profile.components[componentName].buyPrice

						local componentValue
						if (not pendingComponents[componentName]) then
							componentValue = self.db.profile.components[componentName].buyPrice
						else
							componentValue = self.db.profile.components[componentName].value or 0
						end

						local componentValueContribution = componentCount * componentValue
						unclaimedComponentValue = unclaimedComponentValue - componentValueContribution
					end

					if (unclaimedComponentValue <= 0) then
						self:DebugMisc("REMOVING NO UNCLAIMED: "..skillName)
						-- d. if no unclaimedComponentValue, then product is done
						pendingProducts[profession][skillName] = nil
					else
						-- e. add product to pendingComponents for components with adjustable value
						local numAdjustable = 0
						for componentName, componentCount in pairs(product.components) do
							-- REVIEW: perf - doing this lookup twice per component (relative to c.)
							if (pendingComponents[componentName]) then
								if (not pendingComponents[componentName][profession]) then
									pendingComponents[componentName][profession] = {}
								end
								pendingComponents[componentName][profession][skillName] = componentCount
								numAdjustable = numAdjustable + 1
								CTP = CTP + 1
							end
						end

						-- f. update unclaimedValue and adjustableComponents for this skill
						pendingProducts[profession][skillName].unclaimedValue = unclaimedComponentValue
						UV = UV + 1
						adjustableComponents[profession][skillName] = numAdjustable
						AC = AC + 1
					end
				end
			end
		end

		-- TEST4. enumerate products pending
		self:DebugMisc("PC="..PC)
		self:DebugMisc("PP="..PP)
		self:DebugMisc("CTP="..CTP)
		self:DebugMisc("UV="..UV)
		self:DebugMisc("AC="..AC)
local AC2 = 0
local AC2N = {}
		for profession, professionData in pairs(self.db.profile.professions) do
			for skillName, skillData in pairs(professionData) do
				local numAdjustable = adjustableComponents[profession][skillName]
				if (numAdjustable) then
					AC2N[numAdjustable] = (AC2N[numAdjustable] or 0) + 1
if (numAdjustable == 0) then
	self:DebugMisc("NA0 "..skillName)
end
					AC2 = AC2+1
				end
			end
		end
		for a,b in pairs(AC2N) do
			self:DebugMisc("AC2N["..a.."]="..b)
		end
		self:DebugMisc("AC2="..AC2)

local zzz = 0
		-- 4. adjust value for all "single adjustable component" products
		for profession, professionData in pairs(self.db.profile.professions) do
			for skillName, pendingProductData in pairs(pendingProducts[profession]) do
				local product = professionData[skillName]
if (zzz >= 10) then
	return true
end

				if (adjustableComponents[profession][skillName] == 1) then
					local componentChanged, componentUnchanged, increment = self:UpdateProductComponentValues(profession, professionData, skillName)
					if (componentChanged) then
--zzz=zzz+1
						dataChanged = true
					end

					local componentToUpdate = componentChanged or componentUnchanged

					-- whether it changed or not, this skill has now been processed
					self:UpdateCombinedValuesSkillComponentProcessed(	pendingComponents, pendingProducts, adjustableComponents,
																		profession, skillName, componentToUpdate, increment
																	)
				end
			end
		end
if (zzz > 0) then
	return true
end

		-- TODO iterate up until empty pendingProducts, and then assert no pendingComponents
		local numAdjustableAllowed = 2

		-- 5. adjust value for "multiple adjustable component" products

		-- TODO: loop
local qqq = 0
		--if (true) then
		
		-- TODO: different loop condition
		local nothingChanged
		while (not nothingChanged) do
			nothingChanged = true

--if (qqq >= 10) then
--return dataChanged
--end
qqq=qqq+1

self:DebugMisc("checking pending")
			-- a. find the item with the least unclaimedValue
			local constraint
			--local professionLeastFlexible
			--local skillNameLeastFlexible
			for profession, professionData in pairs(self.db.profile.professions) do
				for skillName, pendingProductData in pairs(pendingProducts[profession]) do
					if (pendingProducts[profession] and pendingProducts[profession][skillName]) then
self:DebugMisc("pending products")
						if ((not constraint) or
							(pendingProducts[profession][skillName].unclaimedValue < pendingProducts[constraint.profession][constraint.skillName].unclaimedValue)) then

							constraint = { profession = profession, skillName = skillName, }
							--professionLeastFlexible = profession
							--skillNameLeastFlexible = skillName
						end
					end
				end
			end

			if (not constraint) then
				break
			end

			constraint.unclaimedValue = pendingProducts[constraint.profession][constraint.skillName].unclaimedValue
			self:DebugMisc("constraint "..constraint.profession..","..constraint.skillName..","..constraint.unclaimedValue)
			constraint.components = {}
			for componentName, componentCount in pairs(self.db.profile.professions[constraint.profession][constraint.skillName].components) do
				if (pendingComponents[componentName]) then
					local incrementAllowed = math.floor(constraint.unclaimedValue / componentCount)
					if (incrementAllowed > 0) then
						constraint.components[componentName] = { count = componentCount, incrementAllowed = incrementAllowed, numConsumingSkills = 0 }
					else
					end
				end
			end

			-- b. enumerate components, finding the least constrained and discarding the blocked ones
			for componentName, componentData in pairs(constraint.components) do
				self:DebugMisc(componentName)
				self:DebugMisc(componentData.count)
				self:DebugMisc("incrementAllowed initially "..componentData.incrementAllowed)

				for profession, pendingProfessionData in pairs(pendingComponents[componentName]) do
					for skillName, pendingSkillData in pairs(pendingProfessionData) do
						if (not componentBlocked) then
							--TODO
							--local additionalConstraint = { profession = profession, skillName = skillName, }
							--additionalConstraint.unclaimedValue = pendingProducts[profession][skillName].unclaimedValue
							--additionalConstraint.count = self.db.profile.professions[profession][skillName].components[componentName]
							--additionalConstraint.incrementAllowed = math.floor(additionalConstraint.unclaimedValue / additionalConstraint.count)
							local unclaimedValue = pendingProducts[profession][skillName].unclaimedValue
							local count = self.db.profile.professions[profession][skillName].components[componentName]
							local incrementAllowed = math.floor(unclaimedValue / count)

							if (incrementAllowed < componentData.incrementAllowed) then
								self:DebugMisc("incrementAllowed set to "..componentData.incrementAllowed.." by "..skillName)
								componentData.incrementAllowed = incrementAllowed
							end

							componentData.numConsumingSkills = componentData.numConsumingSkills + 1
						end
					end
				end

				if (componentData.incrementAllowed < 1) then
					self:DebugMisc("rejecting component "..componentName)
					constraint.components[componentName] = nil
				end
			end

			-- c. find the remaining candidate with the most consuming skills
			local componentChosen
			for componentName, componentConstraintData in pairs(constraint.components) do
				self:DebugMisc("candidate:")
				self:DebugMisc("  "..componentName)
				self:DebugMisc("  "..componentConstraintData.count)
				self:DebugMisc("  "..componentConstraintData.incrementAllowed)
				self:DebugMisc("  "..componentConstraintData.numConsumingSkills)
				if ((not componentChosen) or
					(componentChosen.numConsumingSkills < componentConstraintData.numConsumingSkills)) then

					componentChosen = { name = componentName, numConsumingSkills = componentConstraintData.numConsumingSkills, }
				end
			end

			local skillProcessed = true
			if (componentChosen) then
				local componentConstraintDataChosen = constraint.components[componentChosen.name]

				-- d. increment the chosen component and update data structures
				local oldComponentValue = self:GetComponentCopperValue(componentChosen.name) or 0
				local newComponentValue = oldComponentValue + componentConstraintDataChosen.incrementAllowed
				self:DebugMisc(componentChosen.name)
				self:DebugMisc(oldComponentValue)
				self:DebugMisc(newComponentValue)

				local skillText = self:SkillTextFromSkillName(constraint.profession, constraint.skillName)
				local oldBuyString = self:PrettyCurrency(oldComponentValue)
				local newBuyString = self:PrettyCurrency(newComponentValue)
				self:Print(
					string.format(
						"  "..L["%s: Updating %s value from %s to %s."],
						skillText, self:GetComponentText(componentChosen.name), oldBuyString, newBuyString
					))

				self.db.profile.components[componentChosen.name].value = newComponentValue
				self.db.profile.components[componentChosen.name].valueSrc = constraint.skillName
				dataChanged = true
				nothingChanged = false

				self:UpdateCombinedValuesSkillComponentProcessed(	pendingComponents, pendingProducts, adjustableComponents,
																	constraint.profession, constraint.skillName, componentChosen.name, componentConstraintDataChosen.incrementAllowed
																)
			else
				for componentName, componentCount in pairs(self.db.profile.professions[constraint.profession][constraint.skillName].components) do
					self:DebugMisc(constraint.profession)
					self:DebugMisc(constraint.skillName)
					self:DebugMisc(componentName)
					self:UpdateCombinedValuesSkillComponentProcessed(	pendingComponents, pendingProducts, adjustableComponents,
																		constraint.profession, constraint.skillName, componentName, 0
																	)
				end
			end
		end

		self:DebugMisc("PC="..PC)
		self:DebugMisc("PP="..PP)
		self:DebugMisc("CTP="..CTP)
		self:DebugMisc("UV="..UV)
		self:DebugMisc("AC="..AC)

	end

	self:DebugMisc(time())
	return self:DebugExit("UpdateCombinedValues", dataChanged)
end


--
-- REVIEW: adapters
--

function Capitalist:GetItemCopperBuyPrice(itemName)
	if (self:IsResource(itemName)) then
		return self:GetResourceCopperBuyPrice(itemName)
	else
		local skillName, profession = self:SkillNameFromProductName(itemName)
		return self:CalculateSkillCopperCost(profession, skillName, true)
	end
end

--REVIEW: Remove or formalize statistics information
--Capitalist.hit = 0
--Capitalist.miss = 0
function Capitalist:CalculateSkillCopperCost(profession, skillName, useBuyPriceForResources)
	local cost = costsMemory[skillName]

	if (not cost) then
--		Capitalist.miss = Capitalist.miss + 1
		cost = 0

		for componentName, componentCount in pairs(self.db.profile.professions[profession][skillName].components) do
			local componentCostInCopper

			if (self:IsResource(componentName)) then
				if (useBuyPriceForResources) then
					componentCostInCopper = self:GetResourceCopperBuyPrice(componentName)
				else
					componentCostInCopper = self:GetResourceCopperCost(componentName)
				end
			else
				-- TODO: use all skills, find the cheapest
				local componentSkillName, componentProfession = self:SkillNameFromProductName(componentName)
				componentCostInCopper = self:CalculateSkillCopperCost(componentProfession, componentSkillName, useBuyPriceForResources)
			end

			cost = cost + componentCount * componentCostInCopper
		end

		local countPerSkillUse = self.db.profile.professions[profession][skillName].min
		if (not countPerSkillUse) then
			countPerSkillUse = 1
		end

		cost = cost / countPerSkillUse

		costsMemory[skillName] = cost
	else
--		Capitalist.hit = Capitalist.hit + 1
	end

	return cost
end

function Capitalist:OutputItemCost(itemName)
	local skills = self:SkillsFromProductName(itemName)
	if (skills) then
		for skillName, profession in pairs(skills) do
			local itemText = self:ProductTextFromSkillName(profession, skillName)
			local itemCost = self:CalculateSkillCopperCost(profession, skillName, false)
			local recipeText = self:SkillTextFromSkillName(profession, skillName)
			self:Print(string.format(L["Product %s costs %s to make via %s."], itemText, self:PrettyCurrency(itemCost), recipeText))
		end
	elseif (self.db.profile.components[itemName]) then
		local itemText = self:GetComponentText(itemName)
		local itemCost = self:GetResourceCopperCost(itemName)
		local itemMerchant = self:ResourceGetMerchant(itemName)
		if (itemMerchant) then
			self:Print(string.format(L["Resource %s cost is set at %s, sold by %s."], itemText, self:PrettyCurrency(itemCost), itemMerchant))
		else
			self:Print(string.format(L["Resource %s cost is set at %s."], itemText, self:PrettyCurrency(itemCost)))
		end
	else
		self:Print(string.format(L["Item %s value unknown."], itemName))
	end
end
